;====================================================================
;Function name: read_GASP.pro
;written by Jian Zeng
;2005-10-4
;INPUT:  infile--------GASP data file
;        nx_g----------# of elements for GASP data
;        ny_g----------# of lines for GASP data
;        lat_infile----GASP latitude file
;        lon_infile----GASP longitude file
;output: glat----------latitude for GASP data
;        glon----------longitude for GASP data
;        aod-----------GASP aerosol Optical Depth
;        bsfc----------GOES surface reflectance
;        bch1----------GOES channel 1 reflectance
;        bcld----------cloud index, 0:cloudy, 1:clear
;=====================================================================
@read_GASP_latlon.pro

function read_GASP,infile,lat_infile=lat_infile,lon_infile=lon_infile,$
         glat=glat,glon=glon,nx_g=nx_g,$
         ny_g=ny_g,bsfc=bsfc,bch1=bch1,bcld=bcld,bmos=bmos,bsig=bsig,$
	 bmsk=bmsk,bcls=bcls,baodstd=baodstd,origaod=origaod,good=good,bad=bad
print,'in read_GASP'+infile  
  if keyword_set(lat_infile) gt 0 then read_GASP_latlon,lat_infile,lon_infile,nx_g,ny_g,glat,glon

  pos=strpos(infile,'.aod.gz')
  if strmid(infile,pos-3,3) eq 'all' then begin
  ;print,'---truncated GOES AOD--'
  baod = bytarr(nx_g,ny_g)
  bcls = bytarr(nx_g,ny_g)
  baodstd=baod
  bsfc = baod
  bch1 = bsfc
  bmos = bsfc
  bcld = bsfc
  bsig = bsfc
  bmsk = bsfc
  bcls = bsfc

  openr,1,infile,/comp
    readu,1,baod,bmsk,bcls,baodstd,bsfc,bch1,bmos,bcld,bsig
  close,1
  baod = float(baod) / 100. - 0.5
  origaod=baod
  baodstd = float(baodstd) / 100.
  bsfc = float(bsfc) / 500. - .1
  bch1 = float(bch1) / 600.
  bmos = float(bmos) / 600.
  bsig = float(bsig) / 250.- 0.5
  newmask = bytarr(nx_g,ny_g)
  good = where(bsfc lt 0.15 and baod lt 10. and bsfc gt 0.005 $
              and bch1 gt 0 and bcld eq 1 and bcls gt 15 and $
              baodstd lt 0.3 and bsig gt 0.01,ngood,compl=bad, $
;              bsig gt 0.01,ngood,compl=bad, $
              ncomp=nbad)

  if (nbad gt 0) then baod(bad)=-9999.0
  aod=baod

  endif else begin
  ;print,'---untruncated GOES AOD--'
  aod = lonarr(nx_g,ny_g)
  sig = lonarr(nx_g,ny_g)
  msk = bytarr(nx_g,ny_g)
  openr,ilun,infile,/get_lun,/comp
    readu,ilun,aod
    readu,ilun,sig
    readu,ilun,msk  
    close,ilun
  free_lun,ilun

  aod=float(aod)/1000.
  sig=float(sig)/1000.
  good=where(msk eq 1,ngood,compl=bad,ncompl=nbad)
  aod(bad)=-9999.0

  endelse

  return, aod

end
